#!/usr/bin/gawk -f
#
# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of rt-network utilities package.
#
# 	rt-network is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	rt-network is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with rt-network; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


function _extract(parstr,iptstr) {
	# extract a parameter
        regex = "^(.*[[:blank:]]+)?"parstr"[[:blank:]]+([^[:blank:]]+).*$";
        if (match(iptstr,regex)) return gensub(regex,"\\2","",iptstr);
                else return "";
}

function _stripfw(line)
{
	# strip first word from a line
	return gensub(/^[[:blank:]]*[[:alnum:]]+[[:blank:]]+/,"","",line)
}

function _netmask_b2d(line)
{
	# convert netmask from /xx to /xxx.xxx.xxx.xxx

	# init counter
	lines["count"] = 0
	# save a line to work on. If the line is empty, work on $0
	if (line)
	{
		__line = line
	}
	else
	{
		__line = $0
	}

	# get netmask signs from line. There can be zero or more
	# netmask signs. We must handle all the signs.
	while (match(__line,/(([0-9]{1,3}\.){3}[0-9]{1,3}\/[0-9]{1,2})([^0-9]|$)/,__lines))
	{
		lines[lines["count"]] = __lines[1]
		__line = substr(__line,RSTART+RLENGTH)

		split(lines[lines["count"]],__addr_parts,"/")

		lines[lines["count"],"address"] = __addr_parts[1]
		lines[lines["count"],"netmask"] = __addr_parts[2]
		lines["count"]++
	}

	for (label["b2d"] = 0; label["b2d"] < lines["count"]; label["b2d"]++)
	{
#		print lines[label["b2d"],"address"]"/"lines[label["b2d"],"netmask"]

		premask = lines[label["b2d"],"netmask"]
		mask = ""

		# four cycles by 8 bit each
		for (label["b2d","bytes"] = 0; label["b2d","bytes"] < 4; label["b2d","bytes"]++)
		{
			# zero current byte
			byte = 0

			# construct a byte
			for (label["b2d","bits"] = 0; label["b2d","bits"] < 8; label["b2d","bits"]++)
			{
				# shift byte left
				byte = lshift(byte,1)

				# if there is "unused" bits..
				if (premask > 0)
				{
					# use one
					byte = or(byte,1)
					premask--
				}
			}

			# add a byte to the mask
			mask = mask"."byte
		}

		# strip leading "." 
		mask = substr(mask,2)

		#
		if (line)
		{
			sub(lines[label["b2d"]],lines[label["b2d"],"address"]"/"mask,line)
		}
		else
		{
			sub(lines[label["b2d"]],lines[label["b2d"],"address"]"/"mask,$0)
		}
	}
	delete(lines)
	return line
}

function _netmask_d2b(line)
{
	# convert netmask from /xxx.xxx.xxx.xxx to /xx

	# init counter
	lines["count"] = 0
	# save a line to work on. If the line is empty, work on $0
	if (line)
	{
		__line = line
	}
	else
	{
		__line = $0
	}

	# get netmask signs from line. There can be zero or more
	# netmask signs. We must handle all the signs.
	while (match(__line,/(([0-9]{1,3}\.){3}[0-9]{1,3}\/([0-9]{1,3}\.){3}[0-9]{1,3})([^0-9]|$)/,__lines))
	{
		lines[lines["count"]] = __lines[1]
		__line = substr(__line,RSTART+RLENGTH)

		split(lines[lines["count"]],__addr_parts,"/")

		lines[lines["count"],"address"] = __addr_parts[1]
		lines[lines["count"],"netmask"] = __addr_parts[2]
		lines["count"]++
	}

	for (label["d2b"] = 0; label["d2b"] < lines["count"]; label["d2b"]++)
	{

		premask = lines[label["d2b"],"netmask"]
		mask = 0

		# split xxx.xxx.xxx.xxx into bytes
		split(premask,__bytes,".")

		# for every byte
		for (label["d2b","__bytes"] = 1; label["d2b","__bytes"] < 5; label["d2b","__bytes"]++)
		{
			# while byte > 0
			while (__bytes[label["d2b","__bytes"]] > 0)
			{
				# if the rightmost bit == 1, then increment mask
				if (or(__bytes[label["d2b","__bytes"]],1) == __bytes[label["d2b","__bytes"]])
				{
					mask++
				}
				# shift byte right
				__bytes[label["d2b","__bytes"]] = rshift(__bytes[label["d2b","__bytes"]],1)
			}
		}

		#
		if (line)
		{
			sub(lines[label["d2b"]],lines[label["d2b"],"address"]"/"mask,line)
		}
		else
		{
			sub(lines[label["d2b"]],lines[label["d2b"],"address"]"/"mask,$0)
		}
	}
	delete(lines)
	return line
}

